/* Slitherer, a simple QtWebView-based browser for running the web-based Anaconda
 * Based on minibrowser example from Qt licensed BSD-3-Clause
 *
 * Licensed under the Mozilla Public License Version 2.0
 * Fedora-License-Identifier: MPLv2.0
 * SPDX-2.0-License-Identifier: MPL-2.0
 * SPDX-3.0-License-Identifier: MPL-2.0
 *
 * This is free software.
 * For more information on the license, see LICENSE.
 * For more information on free software,
 * see <https://www.gnu.org/philosophy/free-sw.en.html>.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at <https://mozilla.org/MPL/2.0/>.
 */

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QScreen>
#include <QStyleHints>
#include <QtCore/QCommandLineOption>
#include <QtCore/QCommandLineParser>
#include <QtCore/QUrl>
#include <QtQml/QQmlContext>
#include <QtWebView/QtWebView>

using namespace Qt::StringLiterals;

// Workaround: QtQuick does not expose QUrl::fromUserInput.
class Utils : public QObject
{
	Q_OBJECT
public:
	using QObject::QObject;

	Q_INVOKABLE static QUrl fromUserInput(const QString& userInput);
};

QUrl Utils::fromUserInput(const QString& userInput)
{
	if (!userInput.isEmpty()) {
		if (const QUrl result = QUrl::fromUserInput(userInput); result.isValid())
			return result;
	}
	return QUrl::fromUserInput("about:blank"_L1);
}

#include "main.moc"

int main(int argc, char* argv[])
{
	//! [0]
	QtWebView::initialize();
	QGuiApplication app(argc, argv);
	//! [0]
	QGuiApplication::setDesktopFileName("org.fedoraproject.AnacondaInstaller");
	QGuiApplication::setWindowIcon(QIcon::fromTheme("org.fedoraproject.AnacondaInstaller"));
	QCommandLineParser parser;
	QCoreApplication::setApplicationVersion(QT_VERSION_STR);
	parser.setApplicationDescription(QGuiApplication::applicationDisplayName());
	parser.addHelpOption();
	parser.addVersionOption();
	parser.addPositionalArgument("url"_L1, "The initial URL to open."_L1);
	parser.process(QCoreApplication::arguments());
    const QString initialUrl = parser.positionalArguments().value(0, "http://127.0.0.1/cockpit/@localhost/anaconda-webui/index.html"_L1);

	QQmlApplicationEngine engine;
	QQmlContext* context = engine.rootContext();
	context->setContextProperty("utils"_L1, new Utils(&engine));
	context->setContextProperty("initialUrl"_L1,
	                            Utils::fromUserInput(initialUrl));

	QRect geometry = QGuiApplication::primaryScreen()->availableGeometry();
	if (!QGuiApplication::styleHints()->showIsFullScreen()) {
		const QSize size = geometry.size() * 4 / 5;
		const QSize offset = (geometry.size() - size) / 2;
		const QPoint pos = geometry.topLeft() + QPoint(offset.width(), offset.height());
		geometry = QRect(pos, size);
	}

	engine.setInitialProperties(QVariantMap{ { "x"_L1, geometry.x() },
	                                         { "y"_L1, geometry.y() },
	                                         { "width"_L1, geometry.width() },
	                                         { "height"_L1, geometry.height() } });

	engine.load(QUrl("qrc:/main.qml"_L1));
	if (engine.rootObjects().isEmpty())
		return -1;

	return app.exec();
}
